#!/usr/bin/env bash
. "${TESTS_DIRS}/bash_defs.sh"
# Copyright (C) 2008 Red Hat, Inc.
# Written by Dan Kopecek <dkopecek@redhat.com>
# Tue Feb  5 14:37:39 CET 2008

const E_NOFILE=13
const E_NORECORD=14
const E_NOLIST=15
const E_DIFF=16
const E_NEWLIST=17

# --- Error IDs ------- #
const ID_CANTFINDEXEPATH=1
const ID_CANTFINDPKGTOFILE=2
const ID_CHKLIST_FILECHANGED=3
const ID_CHKLIST_FILENOTFOUND=4
const ID_CHKLIST_NORECORD=5
const ID_CHKLIST_NOTFOUND=6
const ID_CKHLIST_NEW=7
const ID_UPDATESAVAIL=8

function check_env () {
    PROC="`mount | sed -n 's/.*on\s\(.*\)\stype proc.*/\1/p'`"
    if [[ -d "${PROC}" ]]; then
	return ${E_OK}
    else
	if `mount -t proc &> /dev/null`; then
	    if check_env; then
		return ${E_OK}
	    else
		return ${E_FAIL}
	    fi
	fi
    fi
    return ${E_FAIL}
}

function proc_getopt () { # 1 - cmdline, 2 - opt
    cat ${1} | tr '\0' '\t' | sed -n "s/.*${2}\t\([^\t]*\)\t.*/\1/p" && echo
}

function proc_getenv () {
    cat ${1} | tr '\0' '\n' | sed -n "s/^${2}=\(.*\)$/\1/p"
}

# <proc_statuc>
function proc_getuid () {
    sed -n 's/^Uid:\s\([[:digit:]]*\).*/\1/p' "${1}"
}

# <proc_status>
function proc_getgid () {
    sed -n 's/^Gid:\s\([[:digit:]]*\).*/\1/p' "${1}"
}

# <file> <list>
function sha256cmp () {
    if [[ ! -f "${1}" ]]; then
	return ${E_NOFILE}
    else
	local NEW="`sha256file \"${1}\"`"
    fi

    if [[ ! -f "${2}" ]]; then
	echo ${NEW}
	touch "${2}" && return ${E_NEWLIST}
	return ${E_NOLIST}
    fi

    local OLD=$(grep "SHA256(${1})" "${2}" 2> /dev/null)

    if [[ -n "${OLD}" ]]; then
	if [[ "${NEW}" == "${OLD}" ]]; then
	    return ${E_OK}
	else
	    echo ${NEW}
	    return ${E_DIFF}
	fi
    else
	echo ${NEW}
	return ${E_NORECORD}
    fi
}

# <file> <list> <save_flag> [<id>]
function check_sha256 () {
    local NEW
 
    NEW="`sha256cmp \"${1}\" \"${2}\"`"

    case "$?" in
	"${E_NEWLIST}")
	    echo ${NEW} > "${2}"
	    report 'WARNING' $ID_CKHLIST_NEW      "Created new SHA256 list." 
	    ;;
	"${E_NOLIST}")
	    report 'WARNING' $ID_CHKLIST_NOTFOUND "SHA256 list not found."
	    ;;
	"${E_OK}")
	    return ${E_OK}
	    ;;
	"${E_DIFF}")
	    report 'WARNING' $ID_CHKLIST_FILECHANGED "File \"${1}\" changed!"
	    if (( ${3} == 1 )); then
		grep -v "SHA256(${1})" "${2}" > "${2}.new"
		echo ${NEW} >> "${2}.new"
		mv "${2}.new" "${2}"
	    fi
	    return ${E_DIFF}
	    ;;
	"${E_NORECORD}")
	    report 'WARNING' $ID_CHKLIST_NORECORD "No SHA256 record for file \"${1}\""
	    if (( ${3} == 1 )); then
		echo ${NEW} >> "${2}"
	    fi
	    return ${E_NORECORD}
	    ;;
	"${E_NOFILE}")
	    report 'WARNING' $ID_CHKLIST_FILENOTFOUND "File \"${1}\" not found."
	    return ${E_NOFILE}
	    ;;
	*)
	    # FIXME: ...
	    return ${E_FAIL}
	    ;;
    esac
}

########################################

check_env           || test_exit ${E_FAIL} "Internal error: check_env failed"

I=0
# TCP
if (( ${CHECK_TCP} == 1 )); then
    for p in `netstat -nlpt | sed -n 's|.*\s\([[:digit:]]\{1,5\}\)/[[:alnum:]]*.*|\1|p'`; do
	PID[$I]=${p}
	let I=$I+1
    done
fi

# UDP
if (( ${CHECK_UDP} == 1 )); then
    for p in `netstat -nlpu | sed -n 's|.*\s\([[:digit:]]\{1,5\}\)/[[:alnum:]]*.*|\1|p'`; do
	PID[$I]=${p}
	let I=$I+1
    done
fi

# SOCK
if (( ${CHECK_UNIX} == 1 )); then
    for p in `netstat -nlpx | sed -n 's|.*\s\([[:digit:]]\{1,5\}\)/[[:alnum:]]*.*|\1|p'`; do
	PID[$I]=${p}
	let I=$I+1
    done
fi

if (( $I == 0 )); then
    test_exit ${E_OK}
fi

# remove duplicates
TMP=${IFS}
IFS=$'\n'
PID=(`echo "${PID[*]}" | sort -n | uniq`)
IFS=${TMP}

# translate files into pkgnames
I=0
for p in ${PID[@]}; do 
   FILE="`readlink ${PROC}/${p}/exe 2> /dev/null`"

   case "${FILE}" in
   	*.#prelink#.*)
   	   FILE="`echo ${FILE} | sed -n 's|^\(.*\)\.#prelink#\..*$|\1|p'`"
   	;;
   esac

   case "`basename \"${FILE}\"`" in
       python|python-*|php|php-*) # It's a plane! No, it's a mothership of universe peoples!! NO, IT'S P-Y-T-H-O-N (or PHP)!!!
	   WRKD="`proc_getenv \"${PROC}/${p}/environ\" PWD`"
	   FILE="`cat ${PROC}/${p}/cmdline | tr '\0' '\n' | grep -E '(^/)|(\./)' | head -n1`"
	   if [[ -n "${FILE}" ]]; then
	  	case "${FILE}" in
	  		/*)# absolute path
	  		;;
	  		*)
	  		   if [[ -n "${WRKD}" ]]; then
	  			FILE="${WRKD}/${FILE}"
	  		   else
	  		   	report 'WARNING' $ID_CANTFINDEXEPATH "Can't find out location of the script (PID: ${p})"
	  		   	continue
	  		   fi
	  		;;
	  	esac
	   else
		# FIXME
		continue
	   fi
	;;
   esac

   PKG="$(rpmquery -f --qf '%{NAME}' "${FILE}" 2> /dev/null)"

   if (( $? == 0 )); then
       PKGS[$I]=${PKG}
       INFO[$I]="${PKG} ${p}"
       let I=$I+1
   else
       report 'WARNING' $ID_CANTFINDPKGTOFILE "Can't find package that owns ${FILE}"
       report 'HINT'    $ID_CANTFINDPKGTOFILE "The origin of this file is unknown."
   fi
done

# remove duplicates
TMP=${IFS}
IFS=$'\n'
PKGS=(`echo "${PKGS[*]}" | sort | uniq`)
INFO=(`echo "${INFO[*]}" | sort | uniq`)
IFS=${TMP}

if (( ${CHECK_UPDATES} != 0)); then
    YPID=$(cat "${YUM_PIDFILE}" 2> /dev/null)
    RUNP=$(ps -A --no-heading -o pid,comm | sed -n "s/^[[:space:]]*${YPID}[[:space:]]*yum\$/yes/p")

    if [[ -z "${YPID}" || -z "${RUNP}" ]]; then
    	MSG="$(yum -d 1 check-update ${PKGS[@]})"
    	
    	if (( $? == 100 )); then
		report 'WARNING' $ID_UPDATESAVAIL "Updates available: $(echo "${MSG}" | sed -n 's/\([[:alpha:]]*\)\s.*/\1/p' | tr '\n' ' ')"
		report 'HINT'    $ID_UPDATESAVAIL "Update your system to get rid of this message."
    	fi
    	unset MSG
    fi
fi

unset PKGS

#for ((i = 0 ; i < ${#INFO[*]}; i++ )); do
#    PKG=$(echo ${INFO[$i]} | awk '{print $1}')
#    PID=$(echo ${INFO[$i]} | awk '{print $2}')
#
#    FNAME="`echo ${PKG} | tr QWERTYUIOPASDFGHJKLZXCVBNM- qwertyuiopasdfghjklzxcvbnm_`"
#
#    if (( ${CHECK_CONFIG} == 1 )); then
#	if [[ "`type -t CHK_${FNAME}`" == "function" ]]; then
#	    CHK_${FNAME} "${PKG}" "${PROC}/${PID}/"
#	#else
#	#    if [[ -d "${SECTOOL_SCRIPT_DIR}" ]]; then
#	#	for source in `find \"${SECTOOL_SCRIPT_DIR}\" -nowarn -name \'${PKG}.*\' -type f -perm 740 -user root -group root`; do
#	#	    . "${script}"
#	#        done
#	#    fi
#	fi
#    fi
#done

test_exit ${E_OK}
# EOF
